import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:get_it/get_it.dart';
import 'package:moru_payment_gateway/dto/request/access_validate.dart';
import 'package:moru_payment_gateway/repository/payment_handler.dart';
import 'package:moru_payment_gateway/repository/payment_impl.dart';
import 'package:http/http.dart' as http;
import 'package:moru_payment_gateway/presentation/widgets/moru_payment_card.dart';
import 'presentation/bloc/access_validator/validate_access_key_cubit.dart';

// This is our global ServiceLocator
GetIt getIt = GetIt.instance;

void main() {
  getIt.registerLazySingleton<PaymentImpl>(() =>
      PaymentImpl(paymentHandler: PaymentHandler(httpClient: http.Client())));
  runApp(MyApp());
}

class MyApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Moru Payment Gateway',
      theme: ThemeData(
        primarySwatch: Colors.blue,
      ),
      home: MyHomePage(),
    );
  }
}

class MyHomePage extends StatefulWidget {
  @override
  _MyHomePageState createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {
  late ValidateAccessKeyCubit _validateAccessKeyCubit;
  late AccessValidate _accessValidate;
  final String accessKey =
      'b4856fab37c347038c31d8239f8c2157ab1421e50563489c9acec0cba073b637';

  @override
  void initState() {
    super.initState();
    _validateAccessKeyCubit = ValidateAccessKeyCubit();
    _accessValidate = AccessValidate(
        accessKey: accessKey, transactionID: 'test1', amount: 1200);
  }

  @override
  Widget build(BuildContext context) {
    return BlocListener<ValidateAccessKeyCubit, ValidateAccessKeyState>(
      listener: (context, state) {
        if (state is ValidateAccessSuccess) {
          Navigator.of(context).push(
            MaterialPageRoute(
              builder: (context) => MoRuPaymentCard(
                  accessKey: accessKey,
                  validatedAccessRes: state.accessRes,
                  onPressed: (pin) {}),
            ),
          );
        } else {}
      },
      child: Scaffold(
        appBar: AppBar(
          title: Text('Moru Payment Gateway'),
        ),
        body: Center(
          child: Column(
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[
              MaterialButton(
                onPressed: () {
                  _validateAccessKeyCubit.validatedAccessKey(
                      accessValidate: _accessValidate);
                },
                child: Text('Buy Now'),
              )
            ],
          ),
        ),
      ),
    );
  }
}
