import 'package:bloc/bloc.dart';
import 'package:meta/meta.dart';
import 'package:moru_payment_gateway/dto/request/user_validate.dart';
import 'package:moru_payment_gateway/dto/response/validated_user_res.dart';
import 'package:moru_payment_gateway/exception/payment_exception.dart';
import 'package:moru_payment_gateway/main.dart';
import 'package:moru_payment_gateway/repository/payment_impl.dart';

part 'user_validator_state.dart';

class UserValidatorCubit extends Cubit<UserValidatorState> {
  UserValidatorCubit() : super(UserValidatorInitial());

  void validateUser({required UserValidate userValidate}) async {
    try {
      final ValidatedUserRes userValidatedRes =
          await getIt<PaymentImpl>().validateUser(userValidate: userValidate);
      emit(UserValidatorSuccess(validatedUserRes: userValidatedRes));
    } on PaymentException catch (_) {
      switch (PaymentException) {
        case BadRequest:
          emit(UserValidatorFail(failedReason: _.failureMessage));
          break;

        case InvalidOtp:
          emit(UserValidatorFail(failedReason: _.failureMessage));
          break;

        case PinUnset:
          emit(UserValidatorFail(failedReason: _.failureMessage));
          break;

        case OTPExpired:
          emit(UserValidatorFail(failedReason: _.failureMessage));
          break;

        case MultipleWrongOTP:
          emit(UserValidatorFail(failedReason: _.failureMessage));
          break;

        case InvalidGatewayUser:
          emit(UserValidatorFail(failedReason: _.failureMessage));
          break;
        case UnauthorizedException:
          emit(UserValidatorFail(failedReason: _.failureMessage));
      }
    } catch (_) {
      emit(UserValidatorFail(failedReason: "Unable to process your request"));
    }
  }
}
