import 'dart:convert';
import 'dart:io';
import 'package:http/http.dart' as http;
import 'package:moru_payment_gateway/dto/request/access_validate.dart';
import 'package:moru_payment_gateway/dto/request/user_validate.dart';
import 'package:moru_payment_gateway/dto/request/pay_data.dart';
import 'package:moru_payment_gateway/dto/response/validated_access_res.dart';
import 'package:moru_payment_gateway/dto/response/validated_user_res.dart';
import 'package:moru_payment_gateway/exception/payment_exception.dart';
import '../constants.dart';

class PaymentHandler {
  late final http.Client _httpClient;
  final String baseNewsUrl = Constants.BASE_URL;

  PaymentHandler({required httpClient}) : _httpClient = httpClient;

  Future<ValidatedAccessRes> validateAccess(
      {required AccessValidate clientData}) async {
    try {
      final validateAccessApi =
          Uri.parse("${baseNewsUrl + Constants.VALIDATE_ACCESS_KEY}");
      final http.Response response = await _httpClient.post(validateAccessApi,
          body: jsonEncode(clientData));

      final Map<String, dynamic> accessValidatedMap =

      _checkResponseStatusCode(response);

      final ValidatedAccessRes validatedAccessRes =
          ValidatedAccessRes.fromJson(accessValidatedMap);

      return validatedAccessRes;
    } catch (_) {
      throw SocketException('Could not process your request');
    }
  }

  Future<ValidatedUserRes> validateUser(
      {required UserValidate userValidate}) async {
    try {
      final http.Response response =
          await _httpClient.get(Uri.parse(baseNewsUrl));

      final Map<String, dynamic> userValidatedMap =
          _checkResponseStatusCode(response);

      final ValidatedUserRes validatedUserRes =
          ValidatedUserRes.fromJson(userValidatedMap);

      return validatedUserRes;
    } catch (_) {
      throw SocketException('Could not process your request');
    }
  }

  Future<Null>? initiatePay({required PayData payData}) async {
    try {
      final http.Response response =
          await _httpClient.get(Uri.parse(baseNewsUrl));

      final Map<String, dynamic> newsResponseMap =
          _checkResponseStatusCode(response);

      /*final UsBusinessNews usBusinessNews =
      UsBusinessNews.fromJson(newsResponseMap);*/

      //return usBusinessNews;
    } catch (_) {
      throw SocketException('Could not process your request');
    }
  }

  Map<String, dynamic> _checkResponseStatusCode(http.Response response) {
    final Map<String, dynamic> gatewayResMap = jsonDecode(response.body);
    switch (response.statusCode) {
      //ok code
      case 200:
        return gatewayResMap;

      case 400:
        throw BadRequest(gatewayResMap['message']);

      case 9000:
        throw InsufficientBalance(gatewayResMap['message']);

      case 9001:
        throw PinUnset(gatewayResMap['message']);

      case 9002:
        throw InvalidUser(gatewayResMap['message']);

      case 9004:
        throw InvalidOtp(gatewayResMap['message']);

      case 9007:
        throw PaymentAlreadyCompleted(gatewayResMap['message']);

      case 9008:
        throw OTPExpired(gatewayResMap['message']);

      case 401:
        throw UnauthorizedException('Unable to fetch News Api');

      case 403:
        throw UnauthorizedException(
            'Sorry you are unauthorized to access this api');

      default:
        throw UnauthorizedException(
            'Sorry you are unauthorized to access this api');
    }
  }
}
